@@ -21,9 +21,13 @@ from neutron_lib.plugins import directory
 from octavia_lib.api.drivers import data_models as octavia_data_model
 from octavia_lib.api.drivers import driver_lib
 from octavia_lib.common import constants as o_constants
+from oslo_db import exception as odb_exc
 from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
-from ovsdbapp.schema.ovn_northbound import impl_idl as idl_ovn
+from ovsdbapp.schema.ovn_northbound import impl_idl as nb_idl_ovn
+from ovsdbapp.schema.ovn_southbound import impl_idl as sb_idl_ovn
+
+import tenacity
 
 # NOTE(mjozefcz): We need base neutron functionals because we need
 # mechanism driver and l3 plugin.
@@ -37,8 +41,9 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
                          base.BaseLoggingTestCase):
 
     def setUp(self):
-        super(TestOvnOctaviaBase, self).setUp()
-        idl_ovn.OvnNbApiIdlImpl.ovsdb_connection = None
+        super().setUp()
+        nb_idl_ovn.OvnNbApiIdlImpl.ovsdb_connection = None
+        sb_idl_ovn.OvnSbApiIdlImpl.ovsdb_connection = None
         # TODO(mjozefcz): Use octavia listeners to provide needed
         # sockets and modify tests in order to verify if fake
         # listener (status) has received valid value.
@@ -55,17 +60,26 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
         self.fake_neutron_client = mock.MagicMock()
         clients.get_neutron_client = mock.MagicMock()
         clients.get_neutron_client.return_value = self.fake_neutron_client
+        self.fake_neutron_client.show_network = self._mock_show_network
         self.fake_neutron_client.show_subnet = self._mock_show_subnet
         self.fake_neutron_client.list_ports = self._mock_list_ports
         self.fake_neutron_client.show_port = self._mock_show_port
         self.fake_neutron_client.delete_port.return_value = True
         self._local_net_cache = {}
+        self._local_cidr_cache = {}
         self._local_port_cache = {'ports': []}
         self.core_plugin = directory.get_plugin()
 
+    def _mock_show_network(self, network_id):
+        network = {}
+        network['id'] = network_id
+        network['provider:physical_network'] = None
+        return {'network': network}
+
     def _mock_show_subnet(self, subnet_id):
         subnet = {}
         subnet['network_id'] = self._local_net_cache[subnet_id]
+        subnet['cidr'] = self._local_cidr_cache[subnet_id]
         return {'subnet': subnet}
 
     def _mock_list_ports(self, **kwargs):
@@ -93,7 +107,7 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
         return e1, e1_s1
 
     def _create_lb_model(self, vip=None, vip_network_id=None,
-                         vip_port_id=None,
+                         vip_subnet_id=None, vip_port_id=None,
                          admin_state_up=True):
         lb = octavia_data_model.LoadBalancer()
         lb.loadbalancer_id = uuidutils.generate_uuid()
@@ -105,6 +119,8 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
 
         if vip_network_id:
             lb.vip_network_id = vip_network_id
+        if vip_subnet_id:
+            lb.vip_subnet_id = vip_subnet_id
         if vip_port_id:
             lb.vip_port_id = vip_port_id
         lb.admin_state_up = admin_state_up
@@ -146,6 +162,14 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
         m_member.admin_state_up = admin_state_up
         return m_member
 
+    def _create_hm_model(self, pool_id, name, delay, timeout, max_retries,
+                         hm_type, max_retries_down=3, admin_state_up=True):
+        return octavia_data_model.HealthMonitor(
+            admin_state_up=admin_state_up, delay=delay,
+            max_retries=max_retries, max_retries_down=max_retries_down,
+            healthmonitor_id=uuidutils.generate_uuid(),
+            name=name, pool_id=pool_id, type=hm_type, timeout=timeout)
+
     def _create_listener_model(self, loadbalancer_id, pool_id=None,
                                protocol_port=80, protocol=None,
                                admin_state_up=True):
@@ -176,6 +200,11 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
                 external_ids[
                     ovn_const.LB_EXT_IDS_LS_REFS_KEY] = jsonutils.loads(
                         ls_refs)
+            member_status = external_ids.get(ovn_const.OVN_MEMBER_STATUS_KEY)
+            if member_status:
+                external_ids[
+                    ovn_const.OVN_MEMBER_STATUS_KEY] = jsonutils.loads(
+                        member_status)
             lb_dict = {'name': lb.name, 'protocol': lb.protocol,
                        'vips': lb.vips, 'external_ids': external_ids}
             try:
@@ -192,11 +221,10 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
 
     def _validate_loadbalancers(self, expected_lbs):
         observed_lbs = self._get_loadbalancers()
-        # NOTE (mjozefcz): assertItemsEqual works only on first level
-        # of comparison, if dicts inside dics are in diffrent
+        # NOTE (mjozefcz): assertCountEqual works only on first level
+        # of comparison, if dicts inside dicts are in different
         # order it would fail.
-        self.assertEqual(len(expected_lbs),
-                         len(observed_lbs))
+        self.assertEqual(len(expected_lbs), len(observed_lbs))
         for expected_lb in expected_lbs:
             # search for LB with same name and protocol
             found = False
@@ -229,6 +257,11 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
                 return lb_uuid in ls_lbs
         return False
 
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(odb_exc.DBError),
+        wait=tenacity.wait_exponential(),
+        stop=tenacity.stop_after_attempt(3),
+        reraise=True)
     def _create_router(self, name, gw_info=None):
         router = {'router':
                   {'name': name,
@@ -239,12 +272,18 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
         router = self.l3_plugin.create_router(self.context, router)
         return router['id']
 
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(odb_exc.DBError),
+        wait=tenacity.wait_exponential(),
+        stop=tenacity.stop_after_attempt(3),
+        reraise=True)
     def _create_net(self, name, cidr, router_id=None):
         n1 = self._make_network(self.fmt, name, True)
         res = self._create_subnet(self.fmt, n1['network']['id'],
                                   cidr)
         subnet = self.deserialize(self.fmt, res)['subnet']
         self._local_net_cache[subnet['id']] = n1['network']['id']
+        self._local_cidr_cache[subnet['id']] = subnet['cidr']
 
         port = self._make_port(self.fmt, n1['network']['id'])
         if router_id:
@@ -308,7 +347,7 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
                         calls_found.append(expected_status)
                         break
             # Validate if we found all expected calls.
-            self.assertItemsEqual(expected_statuses, calls_found)
+            self.assertCountEqual(expected_statuses, calls_found)
 
     def _wait_for_status_and_validate(self, lb_data, expected_status,
                                       check_call=True):
@@ -316,6 +355,77 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
         expected_lbs = self._make_expected_lbs(lb_data)
         self._validate_loadbalancers(expected_lbs)
 
+    def _create_load_balancer_custom_lr_ls_and_validate(
+        self, admin_state_up=True, create_router=True,
+            force_retry_ls_to_lr_assoc=True):
+
+        self._o_driver_lib.update_loadbalancer_status.reset_mock()
+        r_id = self._create_router('r1') if create_router else None
+
+        net_info = []
+        net_info.append(self._create_net("n1", "10.0.1.0/24", r_id))
+        net_info.append(self._create_net("n2", "10.0.2.0/24", r_id))
+        net_info.append(self._create_net("n3", "10.0.3.0/24", r_id))
+
+        lb_data = {}
+        lb_data['model'] = self._create_lb_model(
+            vip=net_info[0][2],
+            vip_network_id=net_info[0][0],
+            vip_subnet_id=net_info[0][1],
+            vip_port_id=net_info[0][3],
+            admin_state_up=admin_state_up)
+
+        lb_data[ovn_const.LB_EXT_IDS_LR_REF_KEY] = \
+            (ovn_const.LR_REF_KEY_HEADER + r_id)
+        lb_data['vip_net_info'] = net_info[0]
+        lb_data[ovn_const.LB_EXT_IDS_LS_REFS_KEY] = {}
+        lb_data['listeners'] = []
+        lb_data['pools'] = []
+        self._update_ls_refs(lb_data, net_info[0][0])
+        ls = [ovn_const.LR_REF_KEY_HEADER + net[0] for net in net_info]
+
+        if force_retry_ls_to_lr_assoc:
+            ls_foo = copy.deepcopy(ls)
+            ls_foo.append('neutron-foo')
+            self.ovn_driver._ovn_helper._find_ls_for_lr = mock.MagicMock()
+            self.ovn_driver._ovn_helper._find_ls_for_lr.side_effect = \
+                [ls_foo, ls]
+
+        self.ovn_driver.loadbalancer_create(lb_data['model'])
+
+        name = '%s%s' % (ovn_const.LB_VIP_PORT_PREFIX,
+                         lb_data['model'].loadbalancer_id)
+        self.driver.update_port(
+            self.context, net_info[0][3], {'port': {'name': name}})
+
+        if lb_data['model'].admin_state_up:
+            expected_status = {
+                'loadbalancers': [{"id": lb_data['model'].loadbalancer_id,
+                                   "provisioning_status": "ACTIVE",
+                                   "operating_status": o_constants.ONLINE}]
+            }
+        else:
+            expected_status = {
+                'loadbalancers': [{"id": lb_data['model'].loadbalancer_id,
+                                   "provisioning_status": "ACTIVE",
+                                   "operating_status": o_constants.OFFLINE}]
+            }
+        self._wait_for_status_and_validate(lb_data, [expected_status])
+        self.assertTrue(
+            self._is_lb_associated_to_ls(
+                lb_data['model'].loadbalancer_id,
+                ovn_const.LR_REF_KEY_HEADER + net_info[0][0]))
+
+        # NOTE(froyo): Just to check all net connected to lr have a
+        # reference to lb
+        for net_id in ls:
+            self.assertTrue(
+                self._is_lb_associated_to_ls(
+                    lb_data['model'].loadbalancer_id,
+                    net_id))
+
+        return lb_data
+
     def _create_load_balancer_and_validate(self, lb_info,
                                            admin_state_up=True,
                                            only_model=False,
@@ -332,6 +442,7 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
         lb_data['vip_net_info'] = net_info
         lb_data['model'] = self._create_lb_model(vip=net_info[2],
                                                  vip_network_id=net_info[0],
+                                                 vip_subnet_id=net_info[1],
                                                  vip_port_id=net_info[3],
                                                  admin_state_up=admin_state_up)
         lb_data[ovn_const.LB_EXT_IDS_LS_REFS_KEY] = {}
@@ -455,7 +566,7 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
                         'neutron:vip_port_id': vip_net_info[3],
                         'enabled': str(lb_data['model'].admin_state_up)}
         # NOTE(mjozefcz): By default we don't set protocol. We don't know if
-        # listener/pool would be TCP or UDP, so do not set it.
+        # listener/pool would be TCP, UDP or SCTP, so do not set it.
         expected_protocols = set()
 
         # Lets fetch list of L4 protocols defined for this LB.
@@ -499,13 +610,12 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
 
         pool_info = {}
         for p in lb_data.get('pools', []):
+            member_status = {}
             external_ids = _get_lb_field_by_protocol(
                 p.protocol.lower(),
                 field='external_ids')
             p_members = ""
             for m in p.members:
-                if not m.admin_state_up:
-                    continue
                 m_info = 'member_' + m.member_id + '_' + m.address
                 m_info += ":" + str(m.protocol_port)
                 m_info += "_" + str(m.subnet_id)
@@ -525,11 +635,22 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
                                     'neutron-%s' % port['network_id'], 0)
                                 ex['neutron-%s' % port['network_id']] = act + 1
                                 break
+                if p.healthmonitor:
+                    member_status[m.member_id] = o_constants.ONLINE
+                    external_ids[ovn_const.LB_EXT_IDS_HMS_KEY] = \
+                        jsonutils.dumps([p.healthmonitor.healthmonitor_id])
+                else:
+                    if m.admin_state_up:
+                        member_status[m.member_id] = o_constants.NO_MONITOR
+                    else:
+                        member_status[m.member_id] = o_constants.OFFLINE
             pool_key = 'pool_' + p.pool_id
             if not p.admin_state_up:
                 pool_key += ':D'
             external_ids[pool_key] = p_members
             pool_info[p.pool_id] = p_members
+            if member_status:
+                external_ids[ovn_const.OVN_MEMBER_STATUS_KEY] = member_status
 
         for listener in lb_data['listeners']:
             expected_vips = _get_lb_field_by_protocol(
@@ -576,10 +697,14 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
         lb_pools.append(m_pool)
         self._o_driver_lib.update_loadbalancer_status.reset_mock()
         self.ovn_driver.pool_create(m_pool)
+        operating_status = (
+            o_constants.ONLINE
+            if listener_id else o_constants.OFFLINE)
+
         expected_status = {
             'pools': [{'id': m_pool.pool_id,
                        'provisioning_status': 'ACTIVE',
-                       'operating_status': o_constants.OFFLINE}],
+                       'operating_status': operating_status}],
             'loadbalancers': [{'id': m_pool.loadbalancer_id,
                                'provisioning_status': 'ACTIVE'}]
         }
@@ -633,17 +758,36 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
         self.ovn_driver.pool_delete(p)
         lb_data['pools'].remove(p)
         expected_status = []
-        # When a pool is deleted and if it has any members, there are
-        # expected to be deleted.
+
+        # When a pool is deleted and if it has a health_monitor associated or
+        # any members, there are expected to be deleted.
+        if p.healthmonitor:
+            member_statuses = [{"id": m.member_id,
+                                "provisioning_status": o_constants.ACTIVE,
+                                "operating_status": o_constants.NO_MONITOR}
+                               for m in p.members]
+            expected_status.append(
+                {'healthmonitors': [{
+                    "id": p.healthmonitor.healthmonitor_id,
+                    "provisioning_status": o_constants.DELETED,
+                    "operating_status": o_constants.NO_MONITOR}],
+                 'members': member_statuses,
+                 'loadbalancers': [{
+                     "id": p.loadbalancer_id,
+                     "provisioning_status": o_constants.ACTIVE}],
+                 'pools': [{"id": p.pool_id,
+                            "provisioning_status": o_constants.ACTIVE}],
+                 'listeners': []})
         for m in p.members:
             expected_status.append(
                 {'pools': [{"id": p.pool_id,
                             "provisioning_status": o_constants.ACTIVE,
                             "operating_status": o_constants.ONLINE}],
                  'members': [{"id": m.member_id,
-                              "provisioning_status": "DELETED"}],
-                 'loadbalancers': [{"id": p.loadbalancer_id,
-                                    "provisioning_status": "ACTIVE"}],
+                              "provisioning_status": o_constants.DELETED}],
+                 'loadbalancers': [{
+                     "id": p.loadbalancer_id,
+                     "provisioning_status": o_constants.ACTIVE}],
                  'listeners': []})
             self._update_ls_refs(
                 lb_data, self._local_net_cache[m.subnet_id], add_ref=False)
@@ -656,7 +800,8 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
             'pools': [{'id': p.pool_id,
                        'provisioning_status': 'DELETED'}],
             'loadbalancers': [{'id': p.loadbalancer_id,
-                               'provisioning_status': 'ACTIVE'}]
+                               'provisioning_status': 'ACTIVE'}],
+            'listeners': []
         }
         if listener_id:
             pool_dict['listeners'] = [{'id': listener_id,
@@ -688,7 +833,7 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
         return listeners
 
     def _create_member_and_validate(self, lb_data, pool_id, subnet_id,
-                                    network_id, address):
+                                    network_id, address, expected_subnet=None):
         self._o_driver_lib.update_loadbalancer_status.reset_mock()
         pool = self._get_pool_from_lb_data(lb_data, pool_id=pool_id)
         pool_status = {'id': pool.pool_id,
@@ -696,7 +841,15 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
                        'operating_status': o_constants.ONLINE}
 
         m_member = self._create_member_model(pool.pool_id, subnet_id, address)
-        pool.members.append(m_member)
+        # The "expected" member value, which might be different from what
+        # we pass to member_create(), for example, if an expected_subnet
+        # was given.
+        if expected_subnet:
+            e_member = copy.deepcopy(m_member)
+            e_member.subnet_id = expected_subnet
+        else:
+            e_member = m_member
+        pool.members.append(e_member)
 
         self.ovn_driver.member_create(m_member)
         self._update_ls_refs(lb_data, network_id)
@@ -708,7 +861,8 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
         expected_status = {
             'pools': [pool_status],
             'members': [{"id": m_member.member_id,
-                         "provisioning_status": "ACTIVE"}],
+                         "provisioning_status": "ACTIVE",
+                         "operating_status": o_constants.NO_MONITOR}],
             'loadbalancers': [{'id': pool.loadbalancer_id,
                                'provisioning_status': 'ACTIVE'}],
             'listeners': expected_listener_status
@@ -720,12 +874,22 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
             if m.address == member_address:
                 return m
 
-    def _update_member_and_validate(self, lb_data, pool_id, member_address):
+    def _update_member_and_validate(self, lb_data, pool_id, member_address,
+                                    remove_subnet_id=False,
+                                    admin_state_up=True):
         pool = self._get_pool_from_lb_data(lb_data, pool_id=pool_id)
 
         member = self._get_pool_member(pool, member_address)
         self._o_driver_lib.update_loadbalancer_status.reset_mock()
-        self.ovn_driver.member_update(member, member)
+        old_member = copy.deepcopy(member)
+        member.admin_state_up = admin_state_up
+        # NOTE(froyo): In order to test update of member without passing the
+        # subnet_id parameter of the member, just to cover the case when a new
+        # member has been created without passing that argument
+        if remove_subnet_id:
+            old_member.subnet_id = None
+
+        self.ovn_driver.member_update(old_member, member)
         expected_status = {
             'pools': [{'id': pool.pool_id,
                        'provisioning_status': 'ACTIVE'}],
@@ -736,7 +900,8 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
             'listeners': []
         }
         if getattr(member, 'admin_state_up', None):
-            expected_status['members'][0]['operating_status'] = "ONLINE"
+            expected_status['members'][0][
+                'operating_status'] = o_constants.NO_MONITOR
         else:
             expected_status['members'][0]['operating_status'] = "OFFLINE"
         self._wait_for_status_and_validate(lb_data, [expected_status])
@@ -778,7 +943,7 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
                                            check_call=False)
 
     def _delete_member_and_validate(self, lb_data, pool_id, network_id,
-                                    member_address):
+                                    member_address, remove_subnet_id=False):
         pool = self._get_pool_from_lb_data(lb_data, pool_id=pool_id)
         member = self._get_pool_member(pool, member_address)
         pool.members.remove(member)
@@ -789,7 +954,15 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
             pool_status['operating_status'] = o_constants.OFFLINE
 
         self._o_driver_lib.update_loadbalancer_status.reset_mock()
-        self.ovn_driver.member_delete(member)
+
+        # NOTE(froyo): In order to test deletion of member without passing
+        # the subnet_id parameter of the member, just to cover the case when
+        # a new member has been created without passing that argument
+        m_member = copy.deepcopy(member)
+        if remove_subnet_id:
+            m_member.subnet_id = None
+
+        self.ovn_driver.member_delete(m_member)
         expected_status = {
             'pools': [pool_status],
             'members': [{"id": member.member_id,
@@ -801,6 +974,104 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
         self._update_ls_refs(lb_data, network_id, add_ref=False)
         self._wait_for_status_and_validate(lb_data, [expected_status])
 
+    def _create_hm_and_validate(self, lb_data, pool_id, name, delay, timeout,
+                                max_retries, hm_type):
+        self._o_driver_lib.update_loadbalancer_status.reset_mock()
+        pool = self._get_pool_from_lb_data(lb_data, pool_id=pool_id)
+        pool_status = {'id': pool.pool_id,
+                       'provisioning_status': o_constants.ACTIVE,
+                       'operating_status': o_constants.ONLINE}
+
+        m_hm = self._create_hm_model(pool.pool_id, name, delay, timeout,
+                                     max_retries, hm_type)
+        pool.healthmonitor = m_hm
+
+        self.ovn_driver.health_monitor_create(m_hm)
+        pool_listeners = self._get_pool_listeners(lb_data, pool_id)
+        expected_listener_status = [
+            {'id': listener.listener_id,
+             'provisioning_status': o_constants.ACTIVE}
+            for listener in pool_listeners]
+
+        expected_member_status = [
+            {'id': m.member_id, 'provisioning_status': o_constants.ACTIVE,
+             'operating_status': o_constants.ONLINE}
+            for m in pool.members]
+
+        expected_hm_status = {'id': m_hm.healthmonitor_id,
+                              'provisioning_status': o_constants.ACTIVE,
+                              'operating_status': o_constants.ONLINE}
+        expected_status = {
+            'pools': [pool_status],
+            'members': expected_member_status,
+            'loadbalancers': [{'id': pool.loadbalancer_id,
+                               'provisioning_status': o_constants.ACTIVE}],
+            'listeners': expected_listener_status,
+            'healthmonitors': [expected_hm_status]
+        }
+        self._wait_for_status_and_validate(lb_data, [expected_status])
+
+    def _update_hm_and_validate(self, lb_data, pool_id, admin_state_up=None):
+        self._o_driver_lib.update_loadbalancer_status.reset_mock()
+        pool = self._get_pool_from_lb_data(lb_data, pool_id=pool_id)
+        hm = pool.healthmonitor
+        old_hm = copy.deepcopy(hm)
+
+        operating_status = o_constants.ONLINE
+        if admin_state_up is not None:
+            hm.admin_state_up = admin_state_up
+            if not admin_state_up:
+                operating_status = o_constants.OFFLINE
+
+        pool_status = {"id": pool.pool_id,
+                       "provisioning_status": o_constants.ACTIVE}
+
+        self.ovn_driver.health_monitor_update(old_hm, hm)
+
+        expected_hm_status = {'id': hm.healthmonitor_id,
+                              'provisioning_status': o_constants.ACTIVE,
+                              'operating_status': operating_status}
+
+        expected_status = {
+            'pools': [pool_status],
+            'loadbalancers': [{'id': pool.loadbalancer_id,
+                               'provisioning_status': o_constants.ACTIVE}],
+            'healthmonitors': [expected_hm_status]}
+
+        self._wait_for_status_and_validate(lb_data, [expected_status])
+
+    def _delete_hm_and_validate(self, lb_data, pool_id):
+        self._o_driver_lib.update_loadbalancer_status.reset_mock()
+        pool = self._get_pool_from_lb_data(lb_data, pool_id=pool_id)
+        hm = pool.healthmonitor
+        pool.healthmonitor = None
+        pool_status = {'id': pool.pool_id,
+                       'provisioning_status': o_constants.ACTIVE}
+        pool_listeners = self._get_pool_listeners(lb_data, pool_id)
+        expected_listener_status = [
+            {'id': listener.listener_id,
+             'provisioning_status': o_constants.ACTIVE}
+            for listener in pool_listeners]
+
+        self.ovn_driver.health_monitor_delete(hm)
+
+        expected_hm_status = {'id': hm.healthmonitor_id,
+                              'provisioning_status': o_constants.DELETED,
+                              'operating_status': o_constants.NO_MONITOR}
+        expected_member_status = [
+            {'id': m.member_id, 'provisioning_status': o_constants.ACTIVE,
+             'operating_status': o_constants.NO_MONITOR}
+            for m in pool.members]
+        expected_status = {
+            'pools': [pool_status],
+            'loadbalancers': [{"id": pool.loadbalancer_id,
+                               "provisioning_status": o_constants.ACTIVE}],
+            'members': expected_member_status,
+            'listeners': expected_listener_status,
+            'healthmonitors': [expected_hm_status]}
+
+        self._wait_for_status_and_validate(lb_data, [expected_status])
+
     def _create_listener_and_validate(self, lb_data, pool_id=None,
                                       protocol_port=80,
                                       admin_state_up=True, protocol='TCP'):
